/*
 * Copyright (c) 2003-2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "network.h"

GPS_SEND_STR *
setupToSocket(const char *hostname, u_int16_t port)
{
	GPS_SEND_STR *to;
	struct protoent *proto;
	struct hostent *host;

	to = (GPS_SEND_STR *)malloc(sizeof(GPS_SEND_STR));
	if (to == NULL) {
		fprintf(stderr, "%s:%d: cannot allocate memory\n", 
		    __FILE__, __LINE__);
		return NULL;
	}
	proto = getprotobyname("udp");
	if (proto == NULL) {
		fprintf(stderr, "%s:%d: getprotobyname\n", __FILE__, __LINE__);
		return NULL;
	}
	to->fd = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
	if (to->fd < 0) {
		fprintf(stderr, "%s:%d: socket/udp: %s\n", __FILE__, __LINE__, 
		    strerror(errno));
		free(to);
		return NULL;
	}

	host = gethostbyname(hostname);
	if (host == NULL) {
		fprintf(stderr, "%s:%d: %s: %s\n", __FILE__, __LINE__, 
		    hostname, hstrerror(h_errno));
		close(to->fd);
		free(to);
		return NULL;
	}
	to->len =  sizeof(to->dst);
	memset(&(to->dst), 0, to->len);
#ifdef __BSD__
	to->dst.sin_len = host->h_length;
#endif
	to->dst.sin_family = AF_INET;
	to->dst.sin_port = htons(port);
	memcpy(&(to->dst.sin_addr.s_addr), host->h_addr, host->h_length);
	
	return to;
}

/*
 * Setup a listener socket
 */
int
setupRcvSocket(u_int16_t port)
{
	int inFd;
	size_t len;
	struct protoent *proto;
	struct sockaddr_in addr;

	/* setup the listner for corrections */
	proto = getprotobyname("udp");
	if (proto == NULL) {
		fprintf(stderr, "%s:%d: getprotobyname: %s\n", __FILE__,
		    __LINE__, strerror(errno));
		return -1;
	}
	inFd = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
	if (inFd < 0) {
		fprintf(stderr, "%s:%d: socket/udp: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		return -1;
	}

	memset(&addr, 0, sizeof(addr));
	len = sizeof(addr);
#ifdef __BSD__
	addr.sin_len = len;
#endif
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(inFd, (struct sockaddr *)&addr, len) < 0) {
		fprintf(stderr, "%s:%d: bind: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		return -1;
	}
	return inFd;
}

#ifdef __sun
/**
 ** Function missing on Solaris 
 **/
const char *
hstrerror(int err)
{
	switch (err) {
	case 0:
		return "Resolver Error 0 (no error)";
		
	case HOST_NOT_FOUND:
		return "Unknown host";

	case TRY_AGAIN:
		return "Host name lookup failure";
		
	case NO_RECOVERY:
		return "Unknown server error";
		
	case NO_DATA:
		return "No address associated with name";
		
	default:
		return "Resolver internal error";
	}
}
#endif
